/* classicon.c - GNOME WM interaction helper functions
 * Copyright (C) 1999 Sam Solon
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "classicon.h"

#define DESKTOP_ENTRY_PATH "=%s=/Desktop Entry/%s"

static gchar*
FindIconInDir (gchar *checkDir,
	       gchar *classFName,
	       gchar *appFName,
	       gchar *classIName,
	       gchar *appIName)
{
  gchar *chkpath = NULL;
  gchar *appiname = NULL;
  gchar *iname = NULL;
  DIR   *curdir;
  struct dirent *dent;
  struct stat statinfo;
  
  /* First try for the class file */

  chkpath = g_concat_dir_and_file (checkDir, classFName);

  if (stat (chkpath, &statinfo) == 0) {
    gchar *cfgpath = g_strdup_printf (DESKTOP_ENTRY_PATH, chkpath,appIName);
    appiname = gnome_config_get_string (cfgpath);

    if (appiname == NULL) {
      g_free (cfgpath);
      cfgpath = g_strdup_printf (DESKTOP_ENTRY_PATH, chkpath, classIName);
      iname = gnome_config_get_string (cfgpath);
    }

    g_free (cfgpath);
  }

  g_free (chkpath);
  
  /*
   * Now see if we can find an appname.desktop if we didn't get an icon
   * for the app name yet.
   */

  if (appiname == NULL) {
    chkpath = g_concat_dir_and_file (checkDir, appFName);

    if (stat (chkpath, &statinfo) == 0) {
      gchar *cfgpath = g_strdup_printf (DESKTOP_ENTRY_PATH,
					chkpath, classIName);

      appiname = gnome_config_get_string (cfgpath);

      g_free (cfgpath);
    }
    g_free (chkpath);
  }

  /*
   * If we found something, return it
   */

  if (appiname || iname) {
    if (appiname) {
      g_free (iname);
      iname = appiname;
    }

    return iname;
  }

  /*
   * We didn't find anything, loop over and recursively call ourselves
   * to handle subdirectories
   */

  curdir = opendir (checkDir);
  g_return_val_if_fail (curdir != NULL, NULL);

  while ((iname == NULL) && (dent = readdir (curdir))) {
    struct stat chkstat;

    /* Don't follow up and self links */

    if ((strcmp (dent->d_name, ".") != 0)
	&&
	(strcmp (dent->d_name, "..") != 0)) {
      gchar* newdir = g_concat_dir_and_file (checkDir, dent->d_name);

      if (stat (newdir, &chkstat) == 0) { /* ignore errors */
	if (S_ISDIR (chkstat.st_mode)) {
	  iname = FindIconInDir (newdir,
				 classFName, appFName,
				 classIName, appIName);
	}
      }
      g_free (newdir);

    }
  }

  closedir (curdir);

  return iname;
}
      
/**
 * icon_for_class
 *
 * @classInfo: an XClassHint structure
 * @iconWidth: width for returned icon
 * @iconHeight:height for returned icon
 * @retPixmap: returned pixmap
 * @retMask:   returned mask
 *
 * @return: TRUE if an icon was found
 *
 * We want to find an icon for the class information passed as @classInfo.
 * This code looks through the $(datadir)/gnome/apps hierarchy for
 * *.desktop file containing the desired icon. Our goal is to try to find the
 * most specific icon while always trying to have something (which is better
 * than the default).
 *
 * The search will proceed by first trying to find <classname>.desktop in a
 * directory. If that file is found an entry <res_name>.icon is looked for
 * in the file (and any plain icon entry is remembered). If <res_name>.icon
 * is defined we're done.
 *
 * If <res_name>.icon isn't found we look for a file <res_name>.desktop and
 * use any "icon" entry in that file. If <res_name>.desktop isn't found, but
 * we had an icon from <res_class>.icon, that is returned.
 *
 * If we still haven't found an icon, we repeat this for each subdirectory
 * until an icon is found.
 *
 * The justification for this seemingly complex search is to support apps
 * that have multiple top level windows (such as Netscape). Creating a "class"
 * based desktop file puts all the definitions in one place.
 *
 * It is also possible to associate an icon with a window "type". For example,
 * Netscape creates a "MailFolder" res_name for a message center window. By
 * defining a MailFolder.desktop any window identified as a MailFolder can
 * be assigned an icon. By adding a "MailFolder.icon" entry to
 * Netscape.desktop (the <res_class>.desktop file) the Netscape message
 * center icon can be given a unique type.
 *
 * The most general rule then is to name the .desktop file with the res_class
 * value, add entries for each top level window as <res_name>.icon, and
 * everything should work nicely.
 */

gboolean
icon_for_class (XClassHint *classInfo,
		gint iconWidth, gint iconHeight,
		GdkPixmap **retPixmap, GdkBitmap **retMask)
{
  gboolean res = FALSE;
  gchar *startdir = gnome_datadir_file ("gnome/apps");
  gchar *ifname= NULL;
  gchar *class_fname= NULL;	/* name of "class" desktop file*/
  gchar *app_fname = NULL;	/* name of "app"desktop file */
  gchar *class_iname= NULL;	/* name of "class" icon */
  gchar *app_iname= NULL;	/* name of "app" icon */

  if ((classInfo->res_name == NULL)
      && (classInfo->res_class == NULL))
    return FALSE;

  /*
   * Build the names
   */

  if (classInfo->res_name) {
    class_fname = g_strdup_printf ("%s.desktop", classInfo->res_class);
    class_iname = g_strdup ("Icon");
  }

  if (classInfo->res_name) {
    app_fname = g_strdup_printf ("%s.desktop", classInfo->res_name);
    app_iname = g_strdup_printf ("%s.Icon", classInfo->res_name);
  }
  
  ifname = FindIconInDir (startdir,
			  class_fname, app_fname,
			  class_iname, app_iname);

  /*
   * If we found a name try to open it and return it
   */

  if (ifname) {
    gchar *iconpath= gnome_pixmap_file (ifname);
    
    if (iconpath) {
      GtkWidget *pixmap;

      pixmap = gnome_pixmap_new_from_file_at_size (iconpath,
						   iconWidth,
						   iconHeight);
      if (pixmap) {
	*retPixmap = GNOME_PIXMAP (pixmap)->pixmap;
	*retMask   = GNOME_PIXMAP (pixmap)->mask;
	gdk_pixmap_ref (*retPixmap);
	gdk_pixmap_ref (*retMask);

	gtk_widget_destroy (pixmap);

	res = TRUE;
      }
    }

    g_free (iconpath);
  }
  
  g_free (startdir);
  g_free (ifname);
  g_free (class_fname);
  g_free (app_fname);
  g_free (class_iname);
  g_free (app_iname);
  
  return res;
}

      
